import React from "react";
import {
  Map,
  TileLayer,
  Marker,
  Popup,
  ZoomControl,
  Polyline
} from "react-leaflet";
import "./css/maps.css";
import {calcDistance} from "./helpers/distance";
import {longitude, latitude, mapWidth, mapHeight, testPath} from "./config/variables";

export default class Leaflet extends React.Component{
  state = {
    latitude: latitude,
    longitude: longitude,
    zoom: 15,
    path: testPath,
    distance: 0,
    pathKey: "",
    pathKeys: [],
    savePathKey: "-"
    }

  componentDidMount(){
    this.fetchPathKeys()
  }

  fetchPath = () => {
    fetch("http://localhost:5000/get-path")
      .then(res => {
        if(res.status >= 400) {
            throw new Error("Server responds with error!");
        }
        return res.json();
      }).then(data => {
        this.setState({path: data[this.state.pathKey]})
        this.getDistance()
      }, err => {
        console.log(err)
      })
  }

  fetchPathKeys = () => {
    fetch("http://localhost:5000/get-path")
      .then(res => {
        if(res.status >= 400) {
            throw new Error("Server responds with error!");
        }
        return res.json();
      }).then(data => {
        this.setState({pathKeys: Object.keys(data)})
      }, err => {
        console.log(err)
      });
  }

  getPath = (event) => {
    this.setState({pathKey: event.target.value})
    this.fetchPath()
  }

  savePath = () => {
    let json = JSON.stringify({
        "path": {
          "name": this.state.savePathKey,
          "coords": this.state.path
        }
      })
    fetch("http://localhost:5000/set-path", {
      method: 'POST',
      headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
      },
      body: json
    }).then(res => res.json)
      .then(data => {
        let pathKeyCopy = this.state.pathKeys.slice();
        pathKeyCopy.push(this.state.savePathKey);
        this.setState({pathKeys: pathKeyCopy})
      })
  }

  setSavePathKey = (event) => {
    this.setState({savePathKey: event.target.value})
  }

  newPathPoint = event => {
    const { lat, lng } = event.latlng
    let pathCopy = this.state.path.slice();
    pathCopy.push([lat, lng]);
    this.setState({path: pathCopy})
    this.getDistance()
  }

  updateMarker = (idx, event) => {
    let {lat, lng} = event.target.getLatLng();
    let pathCopy = this.state.path.slice();
    pathCopy[idx] = [lat, lng]
    this.setState({path: pathCopy})
    this.getDistance()
  }

  getDistance = () => {
    let dst = calcDistance(this.state.path)
    this.setState({distance: dst})
  }

  clearPath = () => {
    this.setState({path: [], distance: 0})
  }
  
  render() {
    const position = [this.state.latitude, this.state.longitude]
    return (
      <div>
        <div className="map">
          <Map
            center={position}
            zoom={this.state.zoom}
            zoomControl={false}
            style={{width: mapWidth, height: mapHeight}}
            onClick={this.newPathPoint}
          >
              <TileLayer
                attribution='&amp;copy <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
                url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
              />
              <ZoomControl position="bottomright" />

              {this.state.path.map((pos, idx) =>
                <Marker
                  position={pos}
                  draggable
                  key={idx}
                  onDragend={(e) => this.updateMarker(idx, e)}
                >
                  <Popup>
                    {idx}
                  </Popup>
                </Marker>
              )}

              <Polyline color="grey" positions={this.state.path} />
          </Map>
        </div>

        <div className="mapInfo">
          <button onClick={this.clearPath}>Clear path</button>
          <button onClick={this.getDistance}>Calculate Distance</button>

          <button onClick={this.savePath}>save path</button>
          <input type="text" onChange={this.setSavePathKey} value={this.state.savePathKey}/>

          <select name="paths" onChange={this.getPath.bind(this)}>
            {this.state.pathKeys.map((name, idx) =>
              <option value={name} key={idx}>{name}</option>
            )}
          </select>

          <p>
            Distanz: {Math.round(this.state.distance*100)/100} km
          </p>
        </div>
      </div>
    );
  }
}